package oracle.iam.assets.eventhandlers;


import Thor.API.Exceptions.tcAPIException;
import Thor.API.Exceptions.tcColumnNotFoundException;
import Thor.API.Exceptions.tcITResourceNotFoundException;

import java.io.Serializable;

import java.util.HashMap;
import java.util.Hashtable;

import java.util.Set;

import oracle.iam.assets.eventhandlers.exception.ContractorIDPPEventHandlerException;
import oracle.iam.assets.eventhandlers.extuserinfo.service.OIMService;
import oracle.iam.assets.eventhandlers.extuserinfo.utils.DBServiceUtils;
import oracle.iam.assets.eventhandlers.extuserinfo.utils.common.Constants;
import oracle.iam.identity.usermgmt.api.UserManagerConstants;
import oracle.iam.identity.usermgmt.api.UserManagerConstants.AttributeName;
import oracle.iam.platform.Platform;
import oracle.iam.platform.context.ContextAware;
import oracle.iam.platform.entitymgr.EntityManager;
import oracle.iam.platform.entitymgr.vo.Entity;
import oracle.iam.platform.kernel.spi.ConditionalEventHandler;
import oracle.iam.platform.kernel.spi.PostProcessHandler;
import oracle.iam.platform.kernel.vo.AbstractGenericOrchestration;
import oracle.iam.platform.kernel.vo.BulkEventResult;
import oracle.iam.platform.kernel.vo.BulkOrchestration;
import oracle.iam.platform.kernel.vo.EventResult;
import oracle.iam.platform.kernel.vo.Orchestration;
import oracle.iam.platform.kernel.vo.OrchestrationTarget;


/**
 *  This class implements below use case (Post Proc Event Handler).
 *  Retrive Contractor ID from database application and updated in their OIM User Profile.
 *  The contractor database has the following structure
 *  ____________________________________________________
 *  | User ID	| Contractor ID |	First Name	| Last Name |
 *  ----------------------------------------------------
 *
 */
public class ContractorIDPostProcEventHandler implements PostProcessHandler {
    private String userID;
    private String contractorID;

    private Hashtable<String, String> connParams;
    private Hashtable<String, String> inputParams;
    private OIMService oimService;
    private String customUserAttribute;
    private String dbITResource;
    private String oimUserType;


    public ContractorIDPostProcEventHandler() {
        super();
    }

    public EventResult execute(long l, long l1, Orchestration orchestration) {
        LOGGER("Enter ContractorIDPostProcEventHandler#execute()");
        LOGGER("EventID   : " + l);
        LOGGER("ProcessID : " + l1);
        
        
        // LOGIC MOVED FROM isApplicable METHOD
        // ************* STARTS HERE ************** 
        // trigger only if the User type is Contractor
        HashMap<String, Serializable> parameters =
            orchestration.getParameters();
  
        configurationInit();
  
        userID = getParamaterValue(parameters, "User Login");
  
        LOGGER("Retrived User ID" + userID);
  
        String userType =
            getParamaterValue(parameters, AttributeName.EMPTYPE.getId());
  
        LOGGER("User Type = " + userType);
  
        //Execute below steps to update user profile
        if (userType.equalsIgnoreCase(oimUserType)) {
            LOGGER("Executing ContractorIDPostProcEventHandler as User Type is" +
                   userType);
            LOGGER("Exit ContractorIDPostProcEventHandler#isApplicable()");
            // ************* ENDS HERE ************** 
        
            // ORIGINAL EXECUTE METHOD CODE STARTS HERE
            OrchestrationTarget target = orchestration.getTarget();
            String entityType = target.getType();
            String entityId = target.getEntityId();
    
            try {
                //connecting to Contractor DB
                DBServiceUtils conn = new DBServiceUtils();
                LOGGER("Establishing connection to External DB ...");
                conn.establishConnection(connParams);
    
                contractorID = conn.getData(inputParams, userID);
                LOGGER("Retrived contractorID : " + contractorID);
    
                if (contractorID != null) {
    
                    LOGGER("Contractor ID found in DB");
                    LOGGER("Modifying user profile");
    
                    LOGGER("Custom attribute name : " + customUserAttribute);
    
    
                    LOGGER("modify user :" + userID);
    
    
                    HashMap<String, Object> attrs = new HashMap<String, Object>();
                    attrs.put(customUserAttribute, contractorID);
    
                    EntityManager mgr = Platform.getService(EntityManager.class);
                    Entity e;
    
                    LOGGER("Executing Method Modify USer : ");
    
                    e = mgr.modifyEntity(entityType, entityId, attrs);
                    orchestration.addParameter(customUserAttribute, contractorID);
    
                    LOGGER("successfully modified oim user ");
                } else {
                    LOGGER("warning message : No contractor ID found for OIM user ID : " +
                           userID + " in external DB: ");
                }
    
            } catch (ContractorIDPPEventHandlerException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }

        // LOGIC MOVED FROM isApplicable METHOD
        // ************* STARTS HERE ************** 
        } else {
            LOGGER("Not Executing ContractorIDPostProcEventHandler as User Type is " +
                   userType);
            LOGGER("Exit ContractorIDPostProcEventHandler#isApplicable()");
        }
        // ************* ENDS HERE ************** 

        LOGGER("Exit ContractorIDPostProcEventHandler#execute()");

        return new EventResult();
    }

    public BulkEventResult execute(long l, long l1,
                                   BulkOrchestration bulkOrchestration) {
        LOGGER("Enter ContractorIDPostProcEventHandler#BulkEventResult execute()");
        
        configurationInit();
        
        OrchestrationTarget target = bulkOrchestration.getTarget();
        String entityType = target.getType();
        String entityId = target.getEntityId();
        String allEntities[] = target.getAllEntityId();
        
        if(allEntities != null && allEntities.length > 0) {
            LOGGER("allEntities.length: "+allEntities.length);
        }
        else {
            LOGGER("allEntities is null/empty");
        }
        
        LOGGER("entityType: "+entityType);
        LOGGER("entityId: "+entityId);
        
        HashMap<String, Serializable> bulkParams[] = bulkOrchestration.getBulkParameters();
        LOGGER("Bulk Parameters Type: "+bulkOrchestration.getBulkParameters().getClass().getName());
        
        if(bulkParams != null && bulkParams.length > 0) {
            LOGGER("Bulk parameters found!");
            LOGGER("Bulk Parameters Length: "+bulkParams.length);
            
            EntityManager entityMgr = Platform.getService(EntityManager.class);
            Entity entity;
            
            int counter = 0;
            
            System.out.println("==============================================================================");
            for(HashMap<String, Serializable> params : bulkParams) {
                String currEntityId = "";
                if(params != null && !params.isEmpty()) {
                    Set<String> keys = params.keySet();
                    for(String key : keys) {
                        if(params.containsKey(key) && params.get(key) != null) {
                            System.out.println(key + ": " + params.get(key).toString());
                        }
                    }
                }
                
                System.out.println("==============================================================================");
                
                userID = getParamaterValue(params, "User Login");
                LOGGER("Retrived User ID" + userID);
                
                String userType =
                    getParamaterValue(params, AttributeName.EMPTYPE.getId());
                LOGGER("User Type = " + userType);
                
                //Execute below steps to update user profile
                if (userType.equalsIgnoreCase(oimUserType)) {
                    LOGGER("Executing ContractorIDPostProcEventHandler as User Type is" +
                           userType);
                    currEntityId = allEntities[counter];
                    // ************* ENDS HERE ************** 
                
                    // ORIGINAL EXECUTE METHOD CODE STARTS HERE
                    try {
                        //connecting to Contractor DB
                        DBServiceUtils conn = new DBServiceUtils();
                        LOGGER("Establishing connection to External DB ...");
                        conn.establishConnection(connParams);
                
                        contractorID = conn.getData(inputParams, userID);
                        LOGGER("Retrived contractorID : " + contractorID);
                
                        if (contractorID != null) {
                
                            LOGGER("Contractor ID found in DB");
                            LOGGER("Modifying user profile");
                
                            LOGGER("Custom attribute name : " + customUserAttribute);
                
                
                            LOGGER("modify user :" + userID);
                
                
                            HashMap<String, Object> attrs = new HashMap<String, Object>();
                            attrs.put(customUserAttribute, contractorID);
                
                            LOGGER("Executing Method Modify USer : ");
                            
//                            entityMgr.reInitializeDataProviders("");
                            entity = entityMgr.modifyEntity(entityType, currEntityId, attrs);
                            bulkOrchestration.addParameter(customUserAttribute, contractorID);
                
                            LOGGER("successfully modified oim user ");
                        } else {
                            LOGGER("warning message : No contractor ID found for OIM user ID : " +
                                   userID + " in external DB: ");
                        }
                
                    } catch (ContractorIDPPEventHandlerException e) {
//                        e.printStackTrace();
                        System.out.println("ContractorIDPPEventHandlerException occurred: " + e.toString());
                    } catch (Exception e) {
//                        e.printStackTrace();
                        System.out.println("Exception occurred: " + e.toString());
                    }

                // LOGIC MOVED FROM isApplicable METHOD
                // ************* STARTS HERE ************** 
                } else {
                    LOGGER("Not Executing ContractorIDPostProcEventHandler as User Type is " +
                           userType);
                }
                // ************* ENDS HERE ************** 
                counter++;
            }
        }
        else {
            LOGGER("No bulk parameters found!");
        }
        
        LOGGER("Exit ContractorIDPostProcEventHandler#BulkEventResult execute()");
        return null;
    }

    public void compensate(long l, long l1,
                           AbstractGenericOrchestration abstractGenericOrchestration) {
    }

    public boolean cancel(long l, long l1,
                          AbstractGenericOrchestration abstractGenericOrchestration) {
        LOGGER("ContractorIDPostProcEventHandler#cancel()");
        return false;
    }

    public void initialize(HashMap<String, String> hashMap) {
        LOGGER("Enter ContractorIDPostProcEventHandler#initialize()");

        LOGGER("Exit ContractorIDPostProcEventHandler#initialize()");
    }

    public boolean isApplicable(AbstractGenericOrchestration abstractGenericOrchestration) {
        LOGGER("Enter ContractorIDPostProcEventHandler#isApplicable()");

        // trigger only if the User type is Contractor
        HashMap<String, Serializable> parameters =
            abstractGenericOrchestration.getParameters();

        configurationInit();

        userID = getParamaterValue(parameters, "User Login");

        LOGGER("Retrived User ID" + userID);

        String userType =
            getParamaterValue(parameters, AttributeName.EMPTYPE.getId());

        LOGGER("User Type = " + userType);

        //Execute below steps to update user profile
        if (userType.equalsIgnoreCase(oimUserType)) {
            LOGGER("Executing ContractorIDPostProcEventHandler as User Type is" +
                   userType);
            LOGGER("Exit ContractorIDPostProcEventHandler#isApplicable()");
            return true;

        } else {
            LOGGER("Not Executing ContractorIDPostProcEventHandler as User Type is " +
                   userType);
            LOGGER("Exit ContractorIDPostProcEventHandler#isApplicable()");
            return false;
        }

    }

    private String getParamaterValue(HashMap<String, Serializable> parameters,
                                     String key) {
        LOGGER("Enter ContractorIDPostProcEventHandler#getParamaterValue()");

        String value =
            (parameters.get(key) instanceof ContextAware) ? (String)((ContextAware)parameters.get(key)).getObjectValue() :
            (String)parameters.get(key);

        LOGGER("Key : " + key + "Value " + value);
        LOGGER("Exit ContractorIDPostProcEventHandler#getParamaterValue()");
        return value;
    }

    private void configurationInit() {
        connParams = new Hashtable<String, String>();
        inputParams = new Hashtable<String, String>();

        oimService = new OIMService();

        LOGGER("Enter ContractorIDPostProcEventHandler#configurationInit()");

        LOGGER("Getting Contractor DB Connection Parameter");
        try {


            LOGGER("Getting  LOOKUP and IT resources Parameter");

            inputParams = oimService.getinputParameters(Constants.OIM_LOOKUP);

            // Fetching IT resource name from Look up table
            LOGGER("Getting DB IT resources from lookup values ");

            oimUserType = inputParams.get(Constants.OIM_USER_TYPE);
            LOGGER("oimUserType taken from Lookup : " + oimUserType);


            if (inputParams.containsKey(Constants.DB_ITResource)) {
                dbITResource = inputParams.get(Constants.DB_ITResource);
                LOGGER("dbITResource taken from Lookup : " + dbITResource);
            } else {
                LOGGER("Not Found any Database IT resource defined in OIM as external DB");
            }

            connParams =
                    oimService.getConnParameters(dbITResource, Constants.OIM_ITRESOURCE_TYPE);

            LOGGER("Getting field that needs to be updated.");

            customUserAttribute =
                    inputParams.get(Constants.OIM_USER_FIELD_TO_UPDATE);

            LOGGER("customUserAttribute taken from Lookup : " +
                   customUserAttribute);

        } catch (tcITResourceNotFoundException e) {
            e.printStackTrace();
        } catch (tcAPIException e) {
            e.printStackTrace();
        } catch (tcColumnNotFoundException e) {
            e.printStackTrace();
        } catch (ContractorIDPPEventHandlerException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        LOGGER("Exit ContractorIDPostProcEventHandler#configurationInit()");
    }

    public static void LOGGER(String msg) {
        System.out.println("LOGGER >> " + msg);
    }
}
