package oracle.iam.assets.eventhandlers.extuserinfo.utils;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Hashtable;

import oracle.iam.assets.eventhandlers.exception.ContractorIDPPEventHandlerException;
import oracle.iam.assets.eventhandlers.extuserinfo.utils.common.Constants;


/**
 *This class is intended to establish the connection with the database and then
 *  get the desired data from the database.
 */
public class DBServiceUtils extends AbstractServiceUtilities {

    private Connection connection;


    public DBServiceUtils() {
    }


    /**
     * This method is used to establish the connection with the database system.
     * @param connParams : This is a HashTable which contains all the database connection
     * parameters.
     * @throws ContractorIDPPEventHandlerException
     */
    @Override
    public void establishConnection(Hashtable<String, String> connParams) throws ContractorIDPPEventHandlerException {
        String driverClass;
        String connectString;
        String userName;
        String password;

        driverClass = connParams.get(Constants.DB_DRIVER_CLASS);
        userName = connParams.get(Constants.DB_USERNAME);
        password = connParams.get(Constants.DB_PASSWORD);
        connectString = connParams.get(Constants.DB_URL);

        try {
            LOGGER("Enter DBServiceUtils#establishConnection");
            Class.forName(driverClass);
            connection =
                    DriverManager.getConnection(connectString, userName, password);
            LOGGER("Database connection successful...");
        } catch (ClassNotFoundException ex) {
            String reason = "Unable to load the driver class.";
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (SQLException ex) {
            String reason = "Could not connect to the database.";
            throw new ContractorIDPPEventHandlerException(reason, ex);
        }

        LOGGER("Exit DBServiceUtils#establishConnection");

    }

    /**
     * This method is used to get the desired data from the database.
     * @param query       : This is the statement that is used to query the database.
     * @return resultSet  :This is the set of result that is returned after querying the database.
     * @throws SQLException
     */
    public ResultSet executeSQL(String query) throws SQLException {
        ResultSet resultSet = null;
        LOGGER("Enter DBServiceUtils#executeSQL()");
        Statement ps = connection.createStatement();

        resultSet = ps.getResultSet();
        LOGGER("Exit DBServiceUtils#executeSQL()");
        return resultSet;
    }

    /**
     * This method is used to get the desired data from the database by forming
     * an appropriate SQL statement.
     * @param inputParams :This is a HashTable which is used to get the values of
     * the parameters required for forming the SQL statement.
     * @param userID      : This is the ID on the basis of which the database is seacrched.
     * @return id         : this function returns the value of the required data field from
     * the requested table.
     * @throws ContractorIDPPEventHandlerException
     */
    @Override
    public String getData(Hashtable<String, String> inputParams,
                          String userID) throws ContractorIDPPEventHandlerException {
        String id = null;
        String queryString;
        try {
            LOGGER("Enter DBServiceUtils#getData()");
            LOGGER("Printing the input parameters hash table....");

            LOGGER("DB.Table.Column.To.Retrieve :-" +
                   inputParams.get("DB.Table.Column.To.Retrieve"));
            LOGGER("field anme :-" + inputParams.get(Constants.DB_FIELDNAME));
            LOGGER("table anme :-" + inputParams.get(Constants.DB_TABLENAME));


            String fieldName = inputParams.get(Constants.DB_FIELDNAME);
            String tableName = inputParams.get(Constants.DB_TABLENAME);
            String returnFieldName =
                inputParams.get(Constants.DB_RETURN_FIELDNAME);
            String searchCriteria = userID;
            searchCriteria = searchCriteria.toUpperCase();

            queryString =
                    "SELECT " + returnFieldName + " FROM " + tableName + " WHERE UPPER(" +
                    fieldName + ") ='" + searchCriteria + "'";
            ResultSet resultSet = null;

            LOGGER("query to Data Base :   " + queryString);

            PreparedStatement ps = connection.prepareStatement(queryString);

            ps.executeQuery();
            resultSet = ps.getResultSet();

            if (resultSet == null) {
                return null;
            }
            if (resultSet.next()) {
                do {
                    id = resultSet.getString(returnFieldName);
                    LOGGER("contractor Id:" + id);
                } while (resultSet.next());
            } else {
                LOGGER("The userID does not exist in the Contractor Table");
            }


        } catch (SQLException ex) {
            String reason = "Error in fetching data from the database.";
            LOGGER(reason + ": "+ex.toString());
            throw new ContractorIDPPEventHandlerException(reason, ex);
        }
        return id;
    }

    public static void LOGGER(String msg) {
        System.out.println("LOGGER >> " + msg);
    }
}
